var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import { createInstance as createCacheInstance } from "localforage";
import { orderBy } from "lodash";
import pLimit from "p-limit";
import { basename, extname } from "path";
import uuid from "uuid";
import { cacheStoreGetOrSet } from "./cache";
import { serializers, } from "./entitySerializers";
import { filterItems } from "./utils";
const sortRecords = (entities, params) => {
    if (!params.sort || !params.sort.field || !params.sort.order) {
        return entities;
    }
    return orderBy(entities, [params.sort.field], [params.sort.order.toLowerCase()]);
};
const filterRecords = (entities, params) => {
    return filterItems(entities, params.filter || {});
};
const paginateRecords = (entities, params) => {
    if (!params.pagination ||
        !params.pagination.page ||
        !params.pagination.perPage) {
        return entities;
    }
    const start = (params.pagination.page - 1) * (params.pagination.perPage || 10);
    return entities.slice(start, start + (params.pagination.perPage || 10));
};
export class BaseProviderFileList {
    constructor(api, { projectId, ref, basePath, serializer }) {
        this.createEntity = (data) => (Object.assign({}, data, { id: uuid() }));
        this.parseEntity = (file) => {
            const content = this.serializer.parse(Buffer.from(file.content, file.encoding).toString("utf8"));
            return Object.assign({ id: basename(file.filePath, extname(file.filePath)) }, content);
        };
        this.stringifyEntity = (entity) => {
            const { id } = entity, data = __rest(entity, ["id"]);
            return JSON.stringify(data, null, 2);
        };
        this.getFilePath = (entityId) => {
            return this.basePath + "/" + entityId + ".json";
        };
        this.projectId = projectId;
        this.ref = ref;
        this.basePath = basePath || "/";
        this.api = api;
        this.cacheStore = createCacheInstance({
            name: "react-admin-git-provider",
            storeName: "filelist",
        });
        this.serializer = new serializers[serializer || "json"]();
    }
    async getList(params = {}) {
        const tree = await this.api.tree(this.projectId, this.ref, this.basePath);
        const limit = pLimit(5);
        const files = await Promise.all(tree.map(async (treeFile) => cacheStoreGetOrSet(this.cacheStore, treeFile.path, () => limit(() => this.api.showFile(this.projectId, this.ref, treeFile.path)), (cached) => cached.blobId === treeFile.id)));
        return {
            data: paginateRecords(filterRecords(sortRecords(files.map(this.parseEntity), params), params), params),
            total: tree.length,
        };
    }
    async getOne(params) {
        return {
            data: this.parseEntity(await this.api.showFile(this.projectId, this.ref, this.getFilePath(params.id))),
        };
    }
    async getMany(params) {
        const manyFiles = await Promise.all(params.ids.map(id => this.api.showFile(this.projectId, this.ref, this.getFilePath(id))));
        return {
            data: manyFiles.map(this.parseEntity),
        };
    }
    async getManyReference(params) {
        return this.getList(Object.assign({}, params, { filter: Object.assign({ [params.target]: params.id }, params.filter) }));
    }
    async create(params) {
        const data = this.createEntity(params.data);
        const filePath = this.getFilePath(data.id);
        await this.api.commit(this.projectId, this.ref, `Create ${filePath}`, [
            {
                action: "create",
                content: this.stringifyEntity(data),
                filePath,
            },
        ]);
        return { data };
    }
    // TODO: check if data are equals, so skip commit
    async update(params) {
        const filePath = this.getFilePath(params.id);
        await this.api.commit(this.projectId, this.ref, `Update ${filePath}`, [
            {
                action: "update",
                content: this.stringifyEntity(params.data),
                filePath: this.getFilePath(params.id),
            },
        ]);
        return {
            data: Object.assign({ id: params.id }, params.data),
        };
    }
    async updateMany(params) {
        const entities = await Promise.all(params.ids.map(async (id) => (await this.getOne({
            id,
        })).data));
        const newEntities = entities.map(entity => (Object.assign({}, entity, params.data)));
        await this.api.commit(this.projectId, this.ref, `Update many in ${this.basePath}`, newEntities.map(entity => ({
            action: "update",
            content: this.stringifyEntity(entity),
            filePath: this.getFilePath(entity.id),
        })));
        return {
            data: newEntities,
        };
    }
    async delete(params) {
        const filePath = this.getFilePath(params.id);
        await this.api.commit(this.projectId, this.ref, `Delete ${filePath}`, [
            {
                action: "delete",
                filePath,
            },
        ]);
        return {
            data: Object.assign({ id: params.id }, params.previousData),
        };
    }
    async deleteMany(params) {
        const actions = params.ids.map(id => ({
            action: "delete",
            filePath: this.getFilePath(id),
        }));
        await this.api.commit(this.projectId, this.ref, `Delete many in ${this.basePath}`, actions);
        return { data: params.ids };
    }
}
