import { Commits, Repositories, RepositoryFiles } from "gitlab";
import { BaseProviderAPI, } from "../../BaseProviderAPI";
import { getToken } from "./authProvider";
export class GitlabProviderAPI extends BaseProviderAPI {
    constructor(gitlabOptions) {
        super();
        this.repositories = new Repositories(Object.assign({}, gitlabOptions, { oauthToken: getToken() }));
        this.repositoryFiles = new RepositoryFiles(Object.assign({}, gitlabOptions, { oauthToken: getToken() }));
        this.commits = new Commits(Object.assign({}, gitlabOptions, { oauthToken: getToken() }));
    }
    async tree(projectId, ref, path) {
        let nextPage = 1;
        let result = [];
        while (nextPage) {
            const data = (await this.repositories.tree(projectId, {
                page: nextPage,
                path,
                ref,
                showPagination: true,
            }));
            result = [...result, ...data.data];
            nextPage = data.pagination.next || 0;
        }
        return result;
    }
    async showFile(projectId, ref, path) {
        return this.repositoryFiles.show(projectId, path, ref);
    }
    async commit(projectId, ref, message, action) {
        await this.commits.create(projectId, ref, message, action, {});
    }
}
