import { Pipelines } from "gitlab";
import { createInstance as createCacheInstance } from "localforage";
import pLimit from "p-limit";
import { cacheStoreGetOrSet } from "../../cache";
import { getToken } from "./authProvider";
export class ProviderPipeline {
    constructor({ gitlabOptions, projectId, ref }) {
        this.projectId = projectId;
        this.ref = ref;
        this.pipelines = new Pipelines(Object.assign({}, gitlabOptions, { oauthToken: getToken() }));
        this.cacheStore = createCacheInstance({
            name: "react-admin-gitlab",
            storeName: "pipelines",
        });
    }
    async getList(params) {
        const pipelineList = (await this.pipelines.all(this.projectId, {
            ref: this.ref,
        }));
        const limit = pLimit(5);
        const pipelines = (await Promise.all(pipelineList.map(pipeline => cacheStoreGetOrSet(this.cacheStore, `${pipeline.id}`, () => limit(() => this.pipelines.show(this.projectId, pipeline.id)), (cached) => {
            return cached.sha === pipeline.sha;
        }))));
        return {
            data: pipelines,
            total: pipelines.length,
        };
    }
    async getOne(params) {
        return {
            data: (await this.pipelines.show(this.projectId, parseInt(params.id, 10))),
        };
    }
    async getMany(params) {
        return {
            data: (await Promise.all(params.ids.map(async (id) => (await this.getOne({ id })).data))),
        };
    }
    async getManyReference(params) {
        return this.getList(Object.assign({}, params, { filter: Object.assign({ [params.target]: params.id }, params.filter) }));
    }
    async create(params) {
        throw new Error("Not available");
        return {
            data: {
                id: "string",
            },
        };
    }
    async update(params) {
        throw new Error("Not available");
        return {
            data: {
                id: "string",
            },
        };
    }
    async updateMany(params) {
        throw new Error("Not available");
        return {
            data: [
                {
                    id: "string",
                },
            ],
        };
    }
    async delete(params) {
        throw new Error("Not available");
        return {
            data: {
                id: "string",
            },
        };
    }
    async deleteMany(params) {
        throw new Error("Not available");
        return {
            data: ["string"],
        };
    }
}
