import querystring from "querystring";
import uuid from "uuid";
const KEY = 'react-admin.gitlab-provider.token';
export function getToken() {
    return window.sessionStorage.getItem(KEY) || undefined;
}
export function setToken(value) {
    return window.sessionStorage.setItem(KEY, value);
}
export function removeToken() {
    return window.sessionStorage.removeItem(KEY);
}
export const initialCheckForToken = () => {
    const qsValue = window.location.hash.replace(/^#\/?/, "");
    if (qsValue) {
        const qs = querystring.parse(qsValue);
        if (typeof qs.access_token === "string") {
            setToken(qs.access_token);
            window.location.hash = "";
        }
    }
};
export const createAuthProvider = (options) => (type, params) => {
    if (type === "AUTH_LOGIN") {
        window.location.href = getOAuthRedirect(options);
    }
    if (type === "AUTH_LOGOUT") {
        removeToken();
        return Promise.resolve();
    }
    if (type === "AUTH_ERROR") {
        return Promise.resolve();
    }
    if (type === "AUTH_CHECK") {
        return getToken() ? Promise.resolve() : Promise.reject();
    }
    return Promise.reject("Unknown method");
};
function getOAuthRedirect(options) {
    const data = querystring.stringify({
        client_id: options.clientId,
        redirect_uri: `${window.location.origin}${window.location.pathname}`,
        response_type: "token",
        scope: "api",
        state: uuid(),
    });
    return `${options.baseUrl || "https://gitlab.com"}/oauth/authorize?${data}`;
}
