const createReactAdminProvider = (provider) => async (type, resource, params) => {
    switch (type) {
        case "GET_LIST":
            return provider.getList(params);
        case "GET_ONE":
            return provider.getOne(params);
        case "GET_MANY":
            return provider.getMany(params);
        case "GET_MANY_REFERENCE":
            return provider.getManyReference(params);
        case "CREATE":
            return provider.create(params);
        case "UPDATE":
            return provider.update(params);
        case "UPDATE_MANY":
            return provider.updateMany(params);
        case "DELETE":
            return provider.delete(params);
        case "DELETE_MANY":
            return provider.deleteMany(params);
        default:
            throw new Error(`Unsupported Data Provider request type ${type}`);
    }
};
export const createDataProvider = (getProvider) => {
    if (typeof getProvider === "function") {
        return async (type, resource, params) => {
            const provider = getProvider({ resource });
            const providerFn = createReactAdminProvider(provider);
            return providerFn(type, resource, params);
        };
    }
    else {
        return createReactAdminProvider(getProvider);
    }
};
