import { every, some } from "lodash";
// Borrowed from https://github.com/marmelab/FakeRest/blob/8397d21b8c8e3a847f80a362dddb1da5cf5a0f5e/src/Collection.js#L5-L81
export function filterItems(items, filter) {
    if (typeof filter === "function") {
        return items.filter(filter);
    }
    if (filter instanceof Object) {
        // turn filter properties to functions
        const filterFunctions = Object.keys(filter).map(key => {
            if (key === "q") {
                const regex = new RegExp(filter.q || "", "i");
                // full-text filter
                return (item) => {
                    for (const itemKey in item) {
                        if (item[itemKey] &&
                            item[itemKey].match &&
                            item[itemKey].match(regex) !== null) {
                            return true;
                        }
                    }
                    return false;
                };
            }
            const value = filter[key];
            if (key.indexOf("_lte") !== -1) {
                // less than or equal
                const realKey = key.replace(/(_lte)$/, "");
                return (item) => item[realKey] <= value;
            }
            if (key.indexOf("_gte") !== -1) {
                // less than or equal
                const realKey = key.replace(/(_gte)$/, "");
                return (item) => item[realKey] >= value;
            }
            if (key.indexOf("_lt") !== -1) {
                // less than or equal
                const realKey = key.replace(/(_lt)$/, "");
                return (item) => item[realKey] < value;
            }
            if (key.indexOf("_gt") !== -1) {
                // less than or equal
                const realKey = key.replace(/(_gt)$/, "");
                return (item) => item[realKey] > value;
            }
            if (Array.isArray(value)) {
                return ((arr, item) => {
                    if (Array.isArray(item[key])) {
                        // array filter and array item value: where all items in values
                        return every(value, v => some(item[key], (itemValue) => itemValue === v));
                    }
                    // where item in values
                    return arr.filter((v) => v === item[key]).length > 0;
                }).bind(null, value);
            }
            return (item) => {
                if (Array.isArray(item[key]) && typeof value === "string") {
                    // simple filter but array item value: where value in item
                    return item[key].indexOf(value) !== -1;
                }
                if (typeof item[key] === "boolean" && typeof value === "string") {
                    // simple filter but boolean item value: boolean where
                    return item[key] === (value === "true" ? true : false);
                }
                // simple filter
                return item[key] === value;
            };
        });
        // only the items matching all filters functions are in (AND logic)
        return items.filter(item => filterFunctions.reduce((selected, filterFunction) => selected && filterFunction(item), true));
    }
    throw new Error("Unsupported filter type");
}
